/*--------------------------------------------------------------------------*/
/*--------------------------- File OPTtypes.h ------------------------------*/
/*--------------------------------------------------------------------------*/
/*--                							  --*/
/*--   Standard types and constants definitions: by changing the	  --*/
/*--   definitions here, the dimension/precision of all the numbers       --*/
/*--   used within the programs can be customized.			  --*/
/*--                							  --*/
/*--                            VERSION 2.30                              --*/
/*--                           24 - 01 - 2001                             --*/
/*--                							  --*/
/*--                 Original Idea and Implementation by:                 --*/
/*--                                                                      --*/
/*--			       Antonio Frangioni       			  --*/
/*--                            Claudio Gentile                           --*/
/*--                                                                      --*/
/*--   			   Operations Research Group			  --*/
/*--			  Dipartimento di Informatica			  --*/
/*--   			     Universita' di Pisa			  --*/
/*--                							  --*/
/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

#ifndef __OPTtypes
 #define __OPTtypes  /* self-identification: #endif at the end of the file */

/*--------------------------------------------------------------------------*/
/*----------------------------- MACROS -------------------------------------*/
/*--------------------------------------------------------------------------*/

/*---------------------------- OPT_TIMERS ----------------------------------*/

#define OPT_TIMERS 0

/* The class OPTtime is defined below to give an abstract interface with the
   different timing routines that are used in different platforms. This is
   needed since time-related functions are one of the less standard parts of
   the C[++] library.

   The value of the OPT_TIMERS constant selects among the different timing
   routines:

   1 = Unix, using sys/times.h;
   2 = timing routines still use sys/times.h, but return wallclock time
       rather than CPU time;
   3 = return the user time obtained with ANSI C clock() function; this
       may result in more accurate running times w.r.t. "1" and "2", but it
       is limited to ~ 72 hours on systems where int are 32bits.

   Any unsupported value would simply make the class to report constant
   zero as the time. */

/*--------------------------------------------------------------------------*/
/*------------------------------ INCLUDES ----------------------------------*/
/*--------------------------------------------------------------------------*/

//#include <values.h>
// limits.h has these constants

/* For informations as
   MAXSHORT , MAXINT , MAXLONG (largest short , regular , long int)
   MAXDOUBLE , MAXFLOAT        (largest double , float )
   DSIGNIF , FSIGNIF           (number of significant bits in double , float)
   */

#include <limits.h>

/* For informations as
   UCHAR_MAX , CHAR_MAX             (largest [unsigned] char)
   USHRT_MAX , UINT_MAX , ULONG_MAX (largest unsigned short , int , long)
   */

/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/

#include <math.h>

/* For the function
   double pow( double b , double e ) */

/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/

#if( ( OPT_TIMERS > 0 ) && ( OPT_TIMERS <= 3 ) )
 #include <time.h>

 #if( OPT_TIMERS <= 2 )
  #include <sys/types.h>
  #include <sys/times.h>
 #endif
#endif

/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/

#include <iostream.h>

/* For istream and the >> operator, used in DfltdSfInpt(). */

/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/

#include <assert.h>

/*--------------------------------------------------------------------------*/
/*--------------------------- NAMESPACE ------------------------------------*/
/*--------------------------------------------------------------------------*/

namespace OPTtypes_di_unipi_it
{

/*--------------------------------------------------------------------------*/
/*----------------------------- NULL ---------------------------------------*/
/*--------------------------------------------------------------------------*/

#ifndef NULL
 #define NULL 0
#endif

/* Curiously enough, some compilers do not always define NULL; for instance,
   sometimes it is defined in stdlib.h. */

/*--------------------------------------------------------------------------*/
/*------------------------- DBLEPS and FLTEPS ------------------------------*/
/*--------------------------------------------------------------------------*/

#ifdef DSIGNIF
 const double DBLEPS = pow( 2.0 , -DSIGNIF );
#else
 const double DBLEPS = 1.1102230246251565e-16;
#endif

#ifdef FSIGNIF
 const double FLTEPS = pow( 2.0 , -FSIGNIF );
#else
 const double FLTEPS = 5.96046448e-08;
#endif

/* These two constants represent the machine precision of doubles and floats,
   i.e. the relative error that one should expect from performing arithmetic
   operations with doubles and floats.
   They can be automatically calculated (as shown above) if the number of
   significant bits for doubles and floats is known: however, not all versions
   of values.h return that information. In the unlucky case that your one is
   among them, you have to provide the numbers by yourself: the ones above
   are for IEEE standard 64- and 32-bits floating point numbers. */

/*--------------------------------------------------------------------------*/
/*---------------------------- TYPES ---------------------------------------*/
/*--------------------------------------------------------------------------*/

#define INT_TYPE  1
#define REAL_TYPE 0

// Some type will have an associated macro, that must be set to INT_TYPE if
// it is short, int, long ... and to REAL_TYPE if it is float, double ...
// so that codes using that type can distinguish the two cases.

/*--------------------------------------------------------------------------*/
/*------------------- General-purpose type definitions ---------------------*/
/*--------------------------------------------------------------------------*/

typedef unsigned char 	BOOL;	        // the smallest integer
typedef BOOL	       *Bool_Vec;	// vector of booleans
typedef Bool_Vec       *Bool_Mat;	// matrix of booleans

typedef const BOOL     cBOOL;           // a read-only boolean
typedef cBOOL         *cBool_Vec;       // read-only array

cBOOL TRUE = 1;
cBOOL FALSE = 0;

/*--------------------------------------------------------------------------*/

typedef unsigned int    Index;	        // Index in a vector ( >= 0 )
typedef Index	       *Index_Set;	// set (array) of indices
typedef Index_Set      *Index_Mat;      // set of set of indices

typedef const Index    cIndex;          // a read-only Index
typedef cIndex        *cIndex_Set;      // read-only array

cIndex InINF = UINT_MAX;                // the largest Index

/*--------------------------------------------------------------------------*/

typedef int             SIndex;         // A Signed Index, for when an index
					// may have a "direction"
typedef SIndex         *SIndex_Set;     // set (array) of s. indices
typedef SIndex_Set     *SIndex_Mat;     // set of set (matrix) of s. indices

typedef const SIndex   cSIndex;         // a read-only Signed Index
typedef cSIndex       *cSIndex_Set;     // read-only array

cSIndex SInINF = INT_MAX; //MAXINT;     // the largest SIndex

/*--------------------------------------------------------------------------*/

typedef long int        INum;           // integer numbers
typedef INum           *IRow;           // vectors of integers
typedef IRow           *IMat;           // matrices (vectors of vectors) of ..

typedef const INum     cINum;           // a read-only integer
typedef cINum         *cIRow;           // read-only array

cINum IntINF = LONG_MAX; //MAXLONG;     // the largest INum

/*--------------------------------------------------------------------------*/

typedef double 		Number;		// "normal" floating point numbers
typedef Number 	       *Row;		// "normal" (fp) array
typedef Row	       *Mat;		// "normal" (fp) matrix

typedef const Number   cNumber;         // a read-only Number
typedef cNumber       *cRow;            // read-only array

cNumber eM = DBLEPS;                    // machine precision
cNumber INF = DBL_MAX; // MAXDOUBLE;    // the largest Number

/*--------------------------------------------------------------------------*/

typedef double          HpNum;          // "finer" floating point numbers
typedef HpNum 	       *HpRow;          // "finer" (fp) array
typedef HpRow	       *HpMat;          // "finer" (fp) matrix

typedef const HpNum    cHpNum;          // a read-only HpNum
typedef cHpNum        *cHpRow;          // read-only array

cHpNum HpeM = DBLEPS;                   // machine precision
cHpNum HpINF = DBL_MAX; // MAXDOUBLE;	// the largest HpNum

/*--------------------------------------------------------------------------*/
/*------------ Type definitions for flow-related algorithms ----------------*/
/*--------------------------------------------------------------------------*/

/*
typedef int             FNumber;        // type of a variable for an arc flow
*/
typedef double          FNumber;        // type of a variable for an arc flow
typedef FNumber        *FRow;           // vector of flows
typedef FRow           *FMat;           // matrix of flows

typedef const FNumber  cFNumber;        // a read-only flow
typedef cFNumber      *cFRow;           // read-only array

/*
cFNumber F_INF = MAXINT;                // the largest FNumber
*/
cFNumber F_INF = DBL_MAX; // MAXDOUBLE; // the largest FNumber

/*
#define Ftype INT_TYPE
*/
#define Ftype REAL_TYPE

#if( Ftype == REAL_TYPE )
 cFNumber F_em = DBLEPS;                // machine precision 
#endif

/*--------------------------------------------------------------------------*/

typedef double          CNumber;        // type of arc flow costs
typedef CNumber        *CRow;           // vector of costs
typedef CRow           *CMat;           // matrix of costs

typedef const CNumber  cCNumber;        // a read-only cost
typedef cCNumber      *cCRow;           // read-only array

cCNumber C_INF = DBL_MAX; // MAXDOUBLE; // the largest CNumber

#define Ctype REAL_TYPE

#if( Ctype == REAL_TYPE )
 cCNumber C_em = DBLEPS;                // machine precision 
#endif

/*--------------------------------------------------------------------------*/

typedef double          FONumber;       // type of the objective function:
                                        // has to hold sums of products of
                                        // FNumber(s) by CNumber(s)

typedef const FONumber cFONumber;       // a read-only o.f. value

cFONumber FO_INF = DBL_MAX; // MAXDOUBLE; // the largest FONumber

#if( ( Ftype == REAL_TYPE ) || ( Ctype == REAL_TYPE ) )
 #define FOtype REAL_TYPE               // the objective function will be
#else                                   // float if either the flows or the
 #define FOtype INT_TYPE                // (or both) are of a float type
#endif

/*--------------------------------------------------------------------------*/

typedef double          MFNumber;       // a Multicommodity flow variable
typedef MFNumber       *MFRow;          // vector of Multicommodity flows
typedef MFRow          *MFMat;          // matrix of Multicommodity flows

typedef const MFNumber cMFNumber;       // a read-only Multicommodity flow
typedef cMFNumber     *cMFRow;          // read-only array

cMFNumber MF_INF = DBL_MAX; // MAXDOUBLE; // the largest MFNumber

#define MFtype REAL_TYPE

#if( MFtype == REAL_TYPE )
 cMFNumber MF_em = DBLEPS;              // machine precision 
#endif

/*--------------------------------------------------------------------------*/
/*----------- Type definitions for subgradient-based algorithms ------------*/
/*--------------------------------------------------------------------------*/

typedef double  	SgNum;          // subgradient entries
typedef SgNum 	       *SgRow;	        // a subgradient
typedef SgRow	       *SgMat;	        // a bundle (set of subgradients)

typedef const SgNum    cSgNum;          // a read-only subgradient entry
typedef cSgNum        *cSgRow;          // a read-only subgradient

const SgNum SgINF = DBL_MAX; // MAXDOUBLE; // the largest subgradient entry

/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/

typedef double          QuNum;          // numbers in Q ( G{i}{T} * G{j} )
typedef QuNum 	       *QuRow;	        // row of Q
typedef QuRow	       *QuMat;	        // Q (itself)

typedef const QuNum    cQuNum;          // a read-only number in Q
typedef cQuNum        *cQuRow;          // a read-only row of Q

cQuNum QuINF = DBL_MAX; // MAXDOUBLE;   // the largest number in Q

/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/

typedef double          LMNum;        // a Lagrangean Multiplier
typedef LMNum 	       *LMRow;	      // a vector of Lagrangean Multipliers
typedef LMRow	       *LMMat;	      // a matrix of Lagrangean Multipliers

typedef const LMNum    cLMNum;        // a read-only Lagrangean Multiplier
typedef cLMNum        *cLMRow;	      // a read-only vector of LMs

cLMNum LMeM = DBLEPS;                 // machine precision
cLMNum LMINF = DBL_MAX; // MAXDOUBLE; // the largest Lagrangean Multiplier

/*--------------------------------------------------------------------------*/
/*--------------------------- OPT_TIMERS -----------------------------------*/
/*--------------------------------------------------------------------------*/

#if( OPT_TIMERS )

class OPTtimers {

 public:  //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

  double u;  // elapsed *user* time, in seconds
  double s;  // elapsed *system* time, in seconds

  inline OPTtimers( void )
  {
   ReSet();
   }

  inline void Start( void )
  {
   #if( OPT_TIMERS == 1 )
    times( &buff );
    t_u = buff.tms_utime;
    t_s = buff.tms_stime;
   #elif( OPT_TIMERS == 2 )
    t_u = times( &buff );
   #elif( OPT_TIMERS == 3 )
    t_u = clock();
   #endif
   }

  inline void Stop( void )
  {
   #if( OPT_TIMERS == 1 )
    times( &buff );
    u += ( double( buff.tms_utime - t_u ) ) / CLK_TCK;
    s += ( double( buff.tms_stime - t_s ) ) / CLK_TCK;
   #elif( OPT_TIMERS == 2 )
    u += ( double( times( &buff ) - t_u ) ) / CLK_TCK;
   #elif( OPT_TIMERS == 3 )
    u += ( double( clock() - t_u ) ) / CLOCKS_PER_SEC;
   #endif
   }

  inline void ReSet( void )
  {
   u = s = 0;
   }

 private:  //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

 #if( ( OPT_TIMERS > 0 ) && ( OPT_TIMERS <= 3 ) )
  clock_t t_u;

  #if( OPT_TIMERS <= 2 )
   struct tms buff;

   #if( OPT_TIMERS == 1 )
    clock_t t_s;
   #endif
  #endif
 #endif

 };  // end( class OPTtimers );

#endif

/*--------------------------------------------------------------------------*/
/*--------------------------- DfltdSfInpt() --------------------------------*/
/*--------------------------------------------------------------------------*/

template<class T>
inline void DfltdSfInpt( istream *iStrm , T &Param , const T Dflt )

/* This function is useful for reading parameters from a istream which may
   not be given, may not be long enough or may contain erroneous things.
   Given a &istream (possibly NULL), DfltdSfInpt() attempts to read a
   parameter of type `T' out of it. If, for any reason, the read operation
   fails, then the parameter is given the default value `Dflt'. Otherwise,
   all the rest of the line (max 127 chars long) up to the nearest newline
   ('\n') carachter is flushed. */
{
 static char buf[ 127 ];

 if( iStrm && (*iStrm) && ( (*iStrm) >> Param ) )
  (*iStrm).getline( buf , 127 );
 else
  Param = Dflt; 
 }

/*--------------------------------------------------------------------------*/

 };  // end( namespace OPTtypes_di_unipi_it )

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

#endif  /* OPTtypes.h included */

/*--------------------------------------------------------------------------*/
/*---------------------- End File OPTtypes.h -------------------------------*/
/*--------------------------------------------------------------------------*/
